/****************************************************************************
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the Qt Mobility Components.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://qt.digia.com/licensing.  For further information
** use the contact form at http://qt.digia.com/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "qmediaresource.h"

#include <QtCore/qsize.h>
#include <QtCore/qurl.h>
#include <QtCore/qvariant.h>

QT_BEGIN_NAMESPACE

/*!
    \class QMediaResource

    \brief The QMediaResource class provides a description of a media resource.
    \inmodule QtMultimediaKit
    \ingroup multimedia
    \since 1.0

    A media resource is composed of a \l {url()}{URL} containing the
    location of the resource and a set of properties that describe the
    format of the resource.  The properties provide a means to assess a
    resource without first attempting to load it, and in situations where
    media be represented by multiple alternative representations provide a
    means to select the appropriate resource.

    Media made available by a remote services can often be available in
    multiple encodings or quality levels, this allows a client to select
    an appropriate resource based on considerations such as codecs supported,
    network bandwidth, and display constraints.  QMediaResource includes
    information such as the \l {mimeType()}{MIME type}, \l {audioCodec()}{audio}
    and \l {videoCodec()}{video} codecs, \l {audioBitRate()}{audio} and
    \l {videoBitRate()}{video} bit rates, and \l {resolution()}{resolution}
    so these constraints and others can be evaluated.

    The only mandatory property of a QMediaResource is the url().

    \sa QMediaContent
*/

/*!
    \typedef QMediaResourceList

    \relates QMediaResource

    Synonym for \c QList<QMediaResource>
*/

/*!
    Constructs a null media resource.
*/
QMediaResource::QMediaResource()
{
}

/*!
    Constructs a media resource with the given \a mimeType from a \a url.
    \since 1.0
*/
QMediaResource::QMediaResource(const QUrl &url, const QString &mimeType)
{
    values.insert(Url, url);
    values.insert(MimeType, mimeType);
}

/*!
    Constructs a media resource with the given \a mimeType from a network \a request.
    \since 1.0
*/
QMediaResource::QMediaResource(const QNetworkRequest &request, const QString &mimeType)
{
    values.insert(Request, QVariant::fromValue(request));
    values.insert(Url, request.url());
    values.insert(MimeType, mimeType);
}

/*!
    Constructs a copy of a media resource \a other.
    \since 1.0
*/
QMediaResource::QMediaResource(const QMediaResource &other)
    : values(other.values)
{
}

/*!
    Assigns the value of \a other to a media resource.
    \since 1.0
*/
QMediaResource &QMediaResource::operator =(const QMediaResource &other)
{
    values = other.values;

    return *this;
}

/*!
    Destroys a media resource.
*/
QMediaResource::~QMediaResource()
{
}


/*!
    Compares a media resource to \a other.

    Returns true if the resources are identical, and false otherwise.
    \since 1.0
*/
bool QMediaResource::operator ==(const QMediaResource &other) const
{
    // Compare requests directly as QNetworkRequests are "custom types".
    foreach (int key, values.keys()) {
        switch (key) {
        case Request:
            if (request() != other.request())
                return false;
        break;
        default:
            if (values.value(key) != other.values.value(key))
                return false;
        }
    }
    return true;
}

/*!
    Compares a media resource to \a other.

    Returns true if they are different, and false otherwise.
    \since 1.0
*/
bool QMediaResource::operator !=(const QMediaResource &other) const
{
    return !(*this == other);
}

/*!
    Identifies if a media resource is null.

    Returns true if the resource is null, and false otherwise.
    \since 1.0
*/
bool QMediaResource::isNull() const
{
    return values.isEmpty();
}

/*!
    Returns the URL of a media resource.
    \since 1.0
*/
QUrl QMediaResource::url() const
{
    return qvariant_cast<QUrl>(values.value(Url));
}

/*!
    Returns the network request associated with this media resource.
    \since 1.0
*/
QNetworkRequest QMediaResource::request() const
{
    if(values.contains(Request))
        return qvariant_cast<QNetworkRequest>(values.value(Request));

    return QNetworkRequest(url());
}

/*!
    Returns the MIME type of a media resource.

    This may be null if the MIME type is unknown.
    \since 1.0
*/
QString QMediaResource::mimeType() const
{
    return qvariant_cast<QString>(values.value(MimeType));
}

/*!
    Returns the language of a media resource as an ISO 639-2 code.

    This may be null if the language is unknown.
    \since 1.0
*/
QString QMediaResource::language() const
{
    return qvariant_cast<QString>(values.value(Language));
}

/*!
    Sets the \a language of a media resource.
    \since 1.0
*/
void QMediaResource::setLanguage(const QString &language)
{
    if (!language.isNull())
        values.insert(Language, language);
    else
        values.remove(Language);
}

/*!
    Returns the audio codec of a media resource.

    This may be null if the media resource does not contain an audio stream, or the codec is
    unknown.
    \since 1.0
*/
QString QMediaResource::audioCodec() const
{
    return qvariant_cast<QString>(values.value(AudioCodec));
}

/*!
    Sets the audio \a codec of a media resource.
    \since 1.0
*/
void QMediaResource::setAudioCodec(const QString &codec)
{
    if (!codec.isNull())
        values.insert(AudioCodec, codec);
    else
        values.remove(AudioCodec);
}

/*!
    Returns the video codec of a media resource.

    This may be null if the media resource does not contain a video stream, or the codec is
    unknonwn.
    \since 1.0
*/
QString QMediaResource::videoCodec() const
{
    return qvariant_cast<QString>(values.value(VideoCodec));
}

/*!
    Sets the video \a codec of media resource.
    \since 1.0
*/
void QMediaResource::setVideoCodec(const QString &codec)
{
    if (!codec.isNull())
        values.insert(VideoCodec, codec);
    else
        values.remove(VideoCodec);
}

/*!
    Returns the size in bytes of a media resource.

    This may be zero if the size is unknown.
    \since 1.0
*/
qint64 QMediaResource::dataSize() const
{
    return qvariant_cast<qint64>(values.value(DataSize));
}

/*!
    Sets the \a size in bytes of a media resource.
    \since 1.0
*/
void QMediaResource::setDataSize(const qint64 size)
{
    if (size != 0)
        values.insert(DataSize, size);
    else
        values.remove(DataSize);
}

/*!
    Returns the bit rate in bits per second of a media resource's audio stream.

    This may be zero if the bit rate is unknown, or the resource contains no audio stream.
    \since 1.0
*/
int QMediaResource::audioBitRate() const
{
    return values.value(AudioBitRate).toInt();
}

/*!
    Sets the bit \a rate in bits per second of a media resource's video stream.
    \since 1.0
*/
void QMediaResource::setAudioBitRate(int rate)
{
    if (rate != 0)
        values.insert(AudioBitRate, rate);
    else
        values.remove(AudioBitRate);
}

/*!
    Returns the audio sample rate of a media resource.

    This may be zero if the sample size is unknown, or the resource contains no audio stream.
    \since 1.0
*/
int QMediaResource::sampleRate() const
{
    return qvariant_cast<int>(values.value(SampleRate));
}

/*!
    Sets the audio \a sampleRate of a media resource.
    \since 1.0
*/
void QMediaResource::setSampleRate(int sampleRate)
{
    if (sampleRate != 0)
        values.insert(SampleRate, sampleRate);
    else
        values.remove(SampleRate);
}

/*!
    Returns the number of audio channels in a media resource.

    This may be zero if the sample size is unknown, or the resource contains no audio stream.
    \since 1.0
*/
int QMediaResource::channelCount() const
{
    return qvariant_cast<int>(values.value(ChannelCount));
}

/*!
    Sets the number of audio \a channels in a media resource.
    \since 1.0
*/
void QMediaResource::setChannelCount(int channels)
{
    if (channels != 0)
        values.insert(ChannelCount, channels);
    else
        values.remove(ChannelCount);
}

/*!
    Returns the bit rate in bits per second of a media resource's video stream.

    This may be zero if the bit rate is unknown, or the resource contains no video stream.
    \since 1.0
*/
int QMediaResource::videoBitRate() const
{
    return values.value(VideoBitRate).toInt();
}

/*!
    Sets the bit \a rate in bits per second of a media resource's video stream.
    \since 1.0
*/
void QMediaResource::setVideoBitRate(int rate)
{
    if (rate != 0)
        values.insert(VideoBitRate, rate);
    else
        values.remove(VideoBitRate);
}

/*!
    Returns the resolution in pixels of a media resource.

    This may be null is the resolution is unknown, or the resource contains no pixel data (i.e. the
    resource is an audio stream.
    \since 1.0
*/
QSize QMediaResource::resolution() const
{
    return qvariant_cast<QSize>(values.value(Resolution));
}

/*!
    Sets the \a resolution in pixels of a media resource.
    \since 1.0
*/
void QMediaResource::setResolution(const QSize &resolution)
{
    if (resolution.width() != -1 || resolution.height() != -1)
        values.insert(Resolution, resolution);
    else
        values.remove(Resolution);
}

/*!
    Sets the \a width and \a height in pixels of a media resource.
    \since 1.0
*/
void QMediaResource::setResolution(int width, int height)
{
    if (width != -1 || height != -1)
        values.insert(Resolution, QSize(width, height));
    else
        values.remove(Resolution);
}
QT_END_NAMESPACE

